// Code generated by "go run string_generate.go"; DO NOT EDIT.

package handlers

import (
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/acl"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/any"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/as112"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/chaos"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/cookie"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/dbfile"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/dbhost"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/dbsqlite"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/drunk"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/ecs"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/empty"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/geoip"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/global"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/id"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/kill"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/log"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/metrics"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/nsid"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/random"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/refuse"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/root"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/sign"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/template"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/tsig"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/unpack"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/url"
	"codeberg.org/miekg/dns/cmd/atomdns/handlers/whoami"
)

// StringToHandler is a map of strings to a handler creation function.
var StringToHandler = map[string]func() Handler{
	"acl":      func() Handler { return new(acl.Acl) },
	"any":      func() Handler { return new(any.Any) },
	"as112":    func() Handler { return new(as112.As112) },
	"chaos":    func() Handler { return new(chaos.Chaos) },
	"cookie":   func() Handler { return new(cookie.Cookie) },
	"dbfile":   func() Handler { return new(dbfile.Dbfile) },
	"dbhost":   func() Handler { return new(dbhost.Dbhost) },
	"dbsqlite": func() Handler { return new(dbsqlite.Dbsqlite) },
	"drunk":    func() Handler { return new(drunk.Drunk) },
	"ecs":      func() Handler { return new(ecs.Ecs) },
	"empty":    func() Handler { return new(empty.Empty) },
	"geoip":    func() Handler { return new(geoip.Geoip) },
	"global":   func() Handler { return new(global.Global) },
	"id":       func() Handler { return new(id.Id) },
	"kill":     func() Handler { return new(kill.Kill) },
	"log":      func() Handler { return new(log.Log) },
	"metrics":  func() Handler { return new(metrics.Metrics) },
	"nsid":     func() Handler { return new(nsid.Nsid) },
	"random":   func() Handler { return new(random.Random) },
	"refuse":   func() Handler { return new(refuse.Refuse) },
	"root":     func() Handler { return new(root.Root) },
	"sign":     func() Handler { return new(sign.Sign) },
	"template": func() Handler { return new(template.Template) },
	"tsig":     func() Handler { return new(tsig.Tsig) },
	"unpack":   func() Handler { return new(unpack.Unpack) },
	"url":      func() Handler { return new(url.Url) },
	"whoami":   func() Handler { return new(whoami.Whoami) },
}
