// Code generated by "go run string_generate.go"; DO NOT EDIT.

package dnsserver

// Handlers is a slices of handler names.
var Handlers = []string{
	"acl",
	"any",
	"as112",
	"chaos",
	"cookie",
	"dbfile",
	"dbhost",
	"dbsqlite",
	"drunk",
	"ecs",
	"empty",
	"geoip",
	"global",
	"id",
	"kill",
	"log",
	"metrics",
	"nsid",
	"random",
	"refuse",
	"root",
	"sign",
	"template",
	"tsig",
	"unpack",
	"url",
	"whoami",
}
